/*
 * signals/oouch.c --
 *
 *	This program demonstrates the C library functions for catching
 *	and raising signals. It also demonstrates how the buffered IO
 *	behaves in combination with signals. In short, be careful!
 */

#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>

static const char progname[] = "oouch";

static void
catch(int signum)
{
    switch (signum) {
    case SIGINT:
        fprintf(stderr,
                "%s: ignoring keyboard interrupt\n", progname);
        break;
    case SIGSEGV:
        fprintf(stderr,
                "%s: ignoring segmentation fault\n", progname);
	/* returning here might cause a loop if the instruction
	   leading to the segmentation fault is restarted... */
	_exit(1);
    default:
        fprintf(stderr,
                "%s: exiting on signal %d\n", progname, signum);
        exit(EXIT_FAILURE);
    }
}

int
main(void)
{
    char c;
    char *p = (char *) 0x42424242;	/* a hopefully invalid address */

    (void) signal(SIGINT,  catch);
    (void) signal(SIGQUIT,  catch);
    (void) signal(SIGSEGV,  catch);

    while ((c = getc(stdin)) != EOF) {
        putc(c, stdout);
    }

    c = *p;			/* forcing a segmentation fault */

    return EXIT_SUCCESS;
}
